@@ -3,4 +3,7 @@ from .CanMessage import CanMessage
 from .CanSignal import CanSignal
 from .CanISignalPdu import CanISignalIPdu
 from .CanStackConfig import CanStackConfig
-from .SomeipPackage import SomeipPackage
\ No newline at end of file
+from .SomeipPackage import SomeipPackage
+from .LinFrame import LinFrame
+from .LinMessage import LinMessage
+from .LinSignal import LinSignal
\ No newline at end of file
