@@ -8,10 +8,13 @@ from concurrent.futures import ThreadPoolExecutor
 import grpc
 import paho.mqtt.client as mqtt
 
+
+
 from . import ZoneSenderData
 
 from .ObjIo import *
 from .ZoneSenderFramework import ZoneSenderFramework
+from .SharedMemory import SharedMemZoneSender
 
 
 class ZoneSenderStone(ZoneSenderFramework):
@@ -26,6 +29,7 @@ class ZoneSenderStone(ZoneSenderFramework):
         self._mqttClient.on_message = self._OnMqttMessage
         self._mqttClient.on_disconnect = self._OnMqttDisconnect
         self._mqttClient.connect(ZoneSenderData.MQTT_BROKER_IP, ZoneSenderData.MQTT_BROKER_PORT)
+        self.shareMem = SharedMemZoneSender()
         self._mqttClient.loop_start()
         
     def Subscribe(self, obj) -> None:
@@ -37,6 +41,9 @@ class ZoneSenderStone(ZoneSenderFramework):
             - ZoneSender.ObjIo.CanSignal\n
             - ZoneSender.ObjIo.CanISignalIPdu\n
             - ZoneSender.ObjIo.SomeipPackage\n
+            - ZoneSender.ObjIo.LinMessage\n
+            - ZoneSender.ObjIo.LinFrame\n
+            - ZoneSender.ObjIo.LinSignal\n
         '''
         obj_ = obj
         if (isinstance(obj_, CanMessage)):
@@ -87,6 +94,7 @@ class ZoneSenderStone(ZoneSenderFramework):
                 print('不能订阅空的 someip service')
                 return
             self._mqttClient.subscribe('zonesender/someipnode/someippackage/{0}'.format(obj_.serviceName))
+            self._mqttClient.subscribe('zonesender/someipnode/someipcalling/{0}'.format(obj_.serviceName))
             self._mqttClient.publish(
                 topic='zonesender/someipnode/subscribe',
                 payload=json.dumps({
@@ -94,7 +102,40 @@ class ZoneSenderStone(ZoneSenderFramework):
                     'service_name': obj_.serviceName
                 }).encode('utf-8')
             )
-    
+        elif isinstance(obj_,LinFrame):
+            self._mqttClient.subscribe('zonesender/linparsernode/frame/{0}/{1}'.format(obj_.channel, obj_.name))
+            self._mqttClient.publish(
+                topic='zonesender/linparsernode/subscribe',
+                payload=json.dumps({
+                    'client_id': self._mqttClient._client_id.decode('utf-8'),
+                    'channel': obj_.channel,
+                    'name': obj_.name,
+                    'type': 'frame',
+                }).encode('utf-8')
+            )
+        elif isinstance(obj_,LinMessage):
+            self._mqttClient.subscribe('zonesender/linparsernode/message/{0}/{1}'.format(obj_.frame_channel, obj_.frame_id))
+            self._mqttClient.publish(
+                topic='zonesender/linparsernode/subscribe',
+                payload=json.dumps({
+                    'client_id': self._mqttClient._client_id.decode('utf-8'),
+                    'channel': obj_.frame_channel,
+                    'name': obj_.frame_id,
+                    'type': 'message',
+                }).encode('utf-8')
+            )
+        elif isinstance(obj_,LinSignal) :
+            self._mqttClient.subscribe('zonesender/linparsernode/signal/{0}/{1}'.format(obj_.channel, obj_.name))
+            self._mqttClient.publish(
+                topic='zonesender/linparsernode/subscribe',
+                payload=json.dumps({
+                    'client_id': self._mqttClient._client_id.decode('utf-8'),
+                    'channel': obj_.channel,
+                    'name': obj_.name,
+                    'type': 'signal',
+                }).encode('utf-8')
+            )
+            
     def UnSubscribe(self, obj) -> None:
         ''' 取消订阅事件，调用该函数后，将不会再 OnXXX 的回调函数中收到相关消息
 
@@ -151,6 +192,7 @@ class ZoneSenderStone(ZoneSenderFramework):
             )
         elif (isinstance(obj_, SomeipPackage)):
             self._mqttClient.unsubscribe('zonesender/someipnode/someippackage/{0}'.format(obj_.serviceName))
+            self._mqttClient.unsubscribe('zonesender/someipnode/someipcalling/{0}'.format(obj_.serviceName))
             self._mqttClient.publish(
                 topic='zonesender/someipnode/unsubscribe',
                 payload=json.dumps({
@@ -158,26 +200,59 @@ class ZoneSenderStone(ZoneSenderFramework):
                     'service_name':obj_.serviceName
                 }).encode('utf-8')
             )
+        elif (isinstance(obj_, LinFrame)):
+            self._mqttClient.unsubscribe('zonesender/linparsernode/frame/{0}/{1}'.format(obj_.channel, obj_.name))
+            self._mqttClient.publish(
+                topic='zonesender/linparsernode/unsubscribe',
+                payload=json.dumps({
+                    'client_id': self._mqttClient._client_id.decode('utf-8'),
+                    'channel': obj_.channel,
+                    'name': obj_.name,
+                    'type': 'frame',
+                }).encode('utf-8')
+            )
+        elif (isinstance(obj_, LinMessage)):
+            self._mqttClient.unsubscribe('zonesender/linparsernode/signal/{0}/{1}'.format(obj_.frame_channel, obj_.frame_id))
+            self._mqttClient.publish(
+                topic='zonesender/linparsernode/unsubscribe',
+                payload=json.dumps({
+                    'client_id': self._mqttClient._client_id.decode('utf-8'),
+                    'channel': obj_.frame_channel,
+                    'name': obj_.frame_id,
+                    'type': 'message',
+                }).encode('utf-8')
+            )
+        elif (isinstance(obj_, LinSignal)):
+            self._mqttClient.unsubscribe('zonesender/linparsernode/signal/{0}/{1}'.format(obj_.channel, obj_.name))
+            self._mqttClient.publish(
+                topic='zonesender/linparsernode/unsubscribe',
+                payload=json.dumps({
+                    'client_id': self._mqttClient._client_id.decode('utf-8'),
+                    'channel': obj_.channel,
+                    'name': obj_.name,
+                    'type': 'signal',
+                }).encode('utf-8')
+            )
 
-    def OnCanFrame(self, timestamp: 'float', can_frame: 'CanFrame') -> None:
+    def OnCanFrame(self, timestamp , can_frame: 'CanFrame') -> None:
         ''' [需要用户重写实现定制功能]\n
         当收到一个 CAN 报文应该做什么，此函数在收到任何订阅过的 CAN 报文的时候触发
 
-        :param timestamp: 收到该数据的时间戳
+        :param timestamp: 收到该数据的时间戳,ns
         :param can_frame: ZoneSender.ObjIo.CanFrame 收到的 CANFrame 对象
         '''
         pass
 
-    def OnCanPdu(self, timestamp: 'float', can_pdu: 'CanISignalIPdu') -> None:
+    def OnCanPdu(self, timestamp, can_pdu: 'CanISignalIPdu') -> None:
         ''' [需要用户重写实现定制功能]\n
         当收到一个 CAN PDU 应该做什么，此函数在收到任何订阅过的 CAN PDU 的时候触发
 
-        :param timestamp: 收到该数据的时间戳
+        :param timestamp: 收到该数据的时间戳,ns
         :param can_pdu: ZoneSender.ObjIo.CanISignalIPdu 收到的 CanISignalIPdu 对象
         '''
         pass
 
-    def OnCanSignal(self, timestamp: 'float', can_signal: 'CanSignal') -> None:
+    def OnCanSignal(self, timestamp, can_signal: 'CanSignal') -> None:
         ''' [需要用户重写实现定制功能]\n
         当收到一个 CAN Signal 应该做什么，此函数在收到任何订阅过的 CAN Signal 的时候触发
 
@@ -186,24 +261,71 @@ class ZoneSenderStone(ZoneSenderFramework):
         '''
         pass
 
-    def OnCanMessage(self, timestamp: 'float', can_message: 'CanMessage') -> None:
+    def OnCanMessage(self, timestamp, can_message: 'CanMessage') -> None:
         ''' [需要用户重写实现定制功能]\n
         当收到一个 CAN Message 应该做什么，此函数在收到任何订阅过的 CAN Message 的时候触发
 
-        :param timestamp: 收到该数据的时间戳
+        :param timestamp: 收到该数据的时间戳,ns
         :param can_message: ZoneSender.ObjIo.CanMessage 收到的 CanMessage 对象
         '''
         pass
 
-    def OnSomeipPackage(self, timestamp: 'float', someip_package: 'SomeipPackage') -> None:
+    def OnSomeipPackage(self, timestamp, someip_package: 'SomeipPackage') -> None:
         ''' [需要用户重写实现定制功能]\n
         当收到一个 SomeipPackage 应该做什么，此函数在收到任何订阅过的 SomeipPackage 的时候触发
 
-        :param timestamp: 收到该数据的时间戳
+        :param timestamp: 收到该数据的时间戳,ns
         :param someip_package: ZoneSender.ObjIo.SomeipPackage 收到的 SomeipPackage 对象
         '''
         pass
 
+    def OnSomeipCalling(self, timestamp, someip_in, someip_out) -> None:
+        ''' [需要用户重写实现定制功能]\n
+        作为 Provider 端调用，当收到 Someip Consumer 的请求做什么\n
+        根据 someip_in 的数据做逻辑判断，给 someip_out.context 赋值实现自定义返回数据\n
+
+        :param timestamp: flaot 收到该数据的时间戳,ns
+        :param someip_in: ZoneSender.ObjIo.SomeipPackage Someip Consumer 端请求的数据
+        :param someip_out: ZoneSender.ObjIp.SomeipPackage 需要填充的数据
+        '''
+        pass
+    
+    def OnSomeipStateChange(self,timestamp,service : 'tuple',state : 'bool') -> None :
+        ''' [需要用户重写实现定制功能]\n
+        订阅的服务状态回调
+        :param timestamp: flaot 收到该数据的时间戳,ns
+        :param service: 服务的具体信息，为一个元组，（service_id,instance_id）
+        :param state: 表明当前服务是否可用的状态
+        '''
+        pass
+
+    def OnLinMessage(self,timestamp, lin_message: 'LinMessage') ->None :
+        ''' [需要用户重写实现定制功能]\n
+        当收到一个 LIN 报文应该做什么，此函数在收到任何订阅过的 LIN 报文的时候触发
+
+        :param timestamp: 收到该数据的时间戳,ns
+        :param lin_message: ZoneSender.ObjIo.LinMessage 收到的 LIN Message 对象
+        '''
+        pass
+    
+    def OnLinFrame(self,timestamp, lin_frame: 'LinFrame') ->None :
+        ''' [需要用户重写实现定制功能]\n
+        当收到一个 LIN 报文应该做什么，此函数在收到任何订阅过的 LIN 报文的时候触发
+
+        :param timestamp: 收到该数据的时间戳,ns
+        :param lin_frame: ZoneSender.ObjIo.LinFrame 收到的 LIN Frame 对象
+        '''
+        pass
+
+    def OnLinSignal(self,timestamp, lin_signal: 'LinSignal') ->None :
+        ''' [需要用户重写实现定制功能]\n
+        当收到一个 LIN 信号应该做什么，此函数在收到任何订阅过的 LIN 信号的时候触发
+
+        :param timestamp: 收到该数据的时间戳,ns
+        :param lin_signal: ZoneSender.ObjIo.LinSignal 收到的 LIN Signal 对象
+        '''
+        pass
+    
     def SendCan(self, obj, **kwargs) -> None:
         '''发送一条CAN报文, 数据将立刻发送
 
@@ -312,20 +434,17 @@ class ZoneSenderStone(ZoneSenderFramework):
         except Exception as e_:
             print(e_)
 
-    def SomeipCallAsync(self, someip_package: 'SomeipPackage') -> None:
+    def SomeipCallAsync(self, someip_package: 'SomeipPackage', *args, **kwargs) -> None:
         ''' 作为 Client调用，请求一个 someip method | get | set
 
         :param someip_package: 要发送的 SomeipPackage
+        :param by: str 设置使用什么参数发送 'context'|'payload'
         :return: None
         '''
         try:
-            d_ = {
-                'sv_name': someip_package.serviceName,
-                'ince_id': someip_package.instanceId,
-                'if_name': someip_package.interfaceName,
-                'if_type': someip_package.interfaceType,
-                'context': someip_package.context
-            }
+            d_ = someip_package.ToDict()
+            d_.update({'by': kwargs.get('by', 'context')})
+            d_.update({'payload': d_['payload'].hex()})
             self._mqttClient.publish(
                 topic='zonesender/someipnode/request/call',
                 payload=json.dumps(d_).encode('utf-8')
@@ -354,26 +473,78 @@ class ZoneSenderStone(ZoneSenderFramework):
         except Exception as e_:
             print(e_)
 
-    def SomeipPublish(self, someip_package: 'SomeipPackage') -> None:
+    def SomeipPublish(self, someip_package: 'SomeipPackage', *args, **kwargs) -> None:
         ''' 作为 Server 调用，发送一次 Notification 或者 Event
 
         :param someip_package: 要发布的 SomeipPackage
+        :param by: str 设置使用什么参数发送 'context'|'payload'
         :return: None
         '''
         try:
-            d_ = {
-                'sv_name': someip_package.serviceName,
-                'ince_id': someip_package.instanceId,
-                'if_name': someip_package.interfaceName,
-                'if_type': someip_package.interfaceType,
-                'context': someip_package.context
-            }
+            d_ = someip_package.ToDict()
+            d_.update({'by': kwargs.get('by', 'context')})
+            d_.update({'payload': d_['payload'].hex()})
             self._mqttClient.publish(
                 topic='zonesender/someipnode/request/publish',
                 payload=json.dumps(d_).encode('utf-8')
             )
         except Exception as e_:
             print(e_)
+    
+    def SetLinData(self,obj) :
+        '''设置LIN报文, 数据将立刻发送
+        :param obj: 要发送的 LIN 对象，可以是如下类型\n
+            - ZoneSender.ObjIo.LinFrame\n
+            - ZoneSender.ObjIo.LinMessage\n
+            - ZoneSender.ObjIo.LinSignal\n
+        '''
+        if isinstance(obj,LinMessage) :
+            d_ = {
+                'channel':obj.frame_channel,
+                'id':obj.frame_id,
+                'name' :obj.name,
+                'data' :obj.frame_data
+            }
+            self._mqttClient.publish(
+                    topic='zonesender/linparsernode/requests/set_frame_data',
+                    payload=json.dumps(d_).encode('utf-8')
+                )
+        elif isinstance(obj,LinFrame) :
+            d_ = {
+                'channel':obj.channel,
+                'id':obj.id,
+                'name' :obj.name,
+                }
+            if obj.context != None:
+                d_['data'] = obj.context
+                d_['encode_type'] = 'unraw'
+            elif obj.context_raw != None:
+                d_['data'] = obj.context_raw
+                d_['encode_type'] = 'raw'
+            elif obj.data != None:
+                d_['data'] = obj.data
+            self._mqttClient.publish(
+                    topic='zonesender/linparsernode/requests/set_frame_data',
+                    payload=json.dumps(d_).encode('utf-8')
+                )
+        elif isinstance(obj,LinSignal) :
+            d_ = {
+                'channel':obj.channel,
+                'name':obj.name
+            }
+            if obj.data_raw != None:
+                d_['data'] = obj.data_raw
+                d_['encode_type'] = 'raw' 
+            elif obj.data != None:
+                d_['data'] = obj.data
+                d_['encode_type'] = 'unraw' 
+            else:
+                print('cant get valid signal value')
+                return
+            self._mqttClient.publish(
+                    topic='zonesender/linparsernode/requests/set_signal_data',
+                    payload=json.dumps(d_).encode('utf-8')
+                )
 
     def _OnMqttMessage(self, client: mqtt.Client, userdata, msg: mqtt.MQTTMessage) -> None:
         '''当收到MQTTMwssage的时候做什么
@@ -383,7 +554,6 @@ class ZoneSenderStone(ZoneSenderFramework):
         :param msg: mqtt.MQTTMessage
         '''
         topic_split_ = msg.topic.split('/')
-        # print(msg.topic)
         # 消息分发
         if (topic_split_[1] == 'canstacknode'):
             recv_d_ = json.loads(msg.payload.decode('utf-8'))
@@ -411,6 +581,24 @@ class ZoneSenderStone(ZoneSenderFramework):
                 # 收到 SomeipPackage
                 # print(recv_d_)
                 self._DealWithSomeipPackage(recv_d_)
+            elif (topic_split_[2] == 'someipcalling'):
+                # 作为 Provider 端收到 Someip Request 后做什么
+                self._DealWithSomeipCalling(recv_d_)
+            elif (topic_split_[2] == 'statechange') :
+                self.OnSomeipStateChange(recv_d_['t'],(recv_d_['srv_id'],recv_d_['inst_id']),recv_d_['state'])   
+        elif topic_split_[1] == 'linstacknode' :
+            pass
+        elif topic_split_[1] == 'linparsernode':
+            recv_d_ = json.loads(msg.payload.decode('utf-8'))
+            if (topic_split_[2] == 'message'):
+                # 收到 CanFrame
+                self._DealWithLinMessage(recv_d_)
+            elif (topic_split_[2] == 'signal'):
+                # 收到CANSIgnal
+                self._DealWithLinSignal(recv_d_)
+            elif (topic_split_[2] == 'frame'):
+                # 收到CANPdu
+                self._DealWithLinFrame(recv_d_)
 
     def _OnMqttConnect(self, client: mqtt.Client, userdata, flags, rc) -> None:
         '''
@@ -420,6 +608,7 @@ class ZoneSenderStone(ZoneSenderFramework):
         self._mqttClient.subscribe('zonesender/out/warn')
         self._mqttClient.subscribe('zonesender/out/error')
         self._mqttClient.subscribe('zonesender/out/debug')
+        self._mqttClient.subscribe('zonesender/someipnode/statechange')
 
     def _OnMqttDisconnect(self, client: mqtt.Client, userdata, rc) -> None:
         '''
@@ -500,19 +689,72 @@ class ZoneSenderStone(ZoneSenderFramework):
         '''
         处理接收到的 SomeipPackage
         '''
-        # print(recv_d)
-        someip_package_ = SomeipPackage(recv_d['sv_name'])
-        someip_package_.srcIp = recv_d['src_ip']
-        someip_package_.srcPort = recv_d['src_port']
-        someip_package_.destIp = recv_d['dest_ip']
-        someip_package_.destPort = recv_d['dest_port']
-        someip_package_.interfaceType = recv_d['type']
-        someip_package_.serviceId = recv_d['sv_id']
-        someip_package_.instanceId = recv_d['ince_id']
-        someip_package_.interfaceId = recv_d['if_id']
-        someip_package_.interfaceName = recv_d['if_name']
-        someip_package_.context = recv_d['context']
+        recv_d['payload'] = bytes.fromhex(recv_d['payload'])
         self._threadPool.submit(
             self.OnSomeipPackage,
-            recv_d['t'], someip_package_
+            recv_d['t'], SomeipPackage.From(recv_d)
+        )
+
+    def _DealWithSomeipCalling(self, recv_d: 'dict') -> None:
+        '''
+        作为 Provider 端调用，处理 Someip Request
+        '''
+        recv_d['payload'] = bytes.fromhex(recv_d['payload'])
+        self._threadPool.submit(
+            self._DealWithSomeipCalling_,
+            recv_d,
+        )
+
+    def _DealWithSomeipCalling_(self, recv_d: 'dict') -> None:
+        '''
+        self._DealWithSomeipCalling 的线程函数
+        '''
+        someip_in_ = SomeipPackage.From(recv_d)
+        someip_out_ = SomeipPackage.From(recv_d)
+        # someip_out_.context = {}
+        self.OnSomeipCalling(recv_d['t'], someip_in_, someip_out_)
+        
+        self.SomeipSetDefaultAnswer(someip_out_)
+
+    def _DealWithLinMessage(self,recv_d:dict) ->None :
+        lin_message_ = LinMessage(
+            channel= recv_d['chl'],
+            frameId= recv_d['id'],
+            data = recv_d['d_real'],
+            dlc= recv_d['dlc'],
+            context= recv_d['d_'],
+            context_raw= recv_d['d_raw']
+        )
+        lin_message_.name = recv_d['name']
+        self._threadPool.submit(
+            self.OnLinMessage,
+            recv_d['t'], lin_message_
+        )
+    
+    def _DealWithLinFrame(self,recv_d:dict) ->None :
+        lin_frame_ = LinFrame(
+            name= recv_d['name'],
+            channel= recv_d['chl'],
+            context= recv_d['d_'],
+            context_raw= recv_d['d_raw']
+        )
+        lin_frame_.id = recv_d['id']
+        lin_frame_.dlc = recv_d['dlc']
+        lin_frame_.data = recv_d['d_real']
+        self._threadPool.submit(
+            self.OnLinFrame,
+            recv_d['t'], lin_frame_
+        )
+    
+    def _DealWithLinSignal(self,recv_d:dict) ->None :
+        lin_signal_ = LinSignal(
+            name= recv_d['name'],
+            channel= recv_d['chl'],
+            data_raw= recv_d['d_raw'],
+            data= recv_d['d_']
+        )
+        lin_signal_.id = recv_d['id']
+        self._threadPool.submit(
+            self.OnLinSignal,
+            recv_d['t'], lin_signal_
         )
